/*
############################################################################
# Written by Dave Hein
# Copyright (c) 2012 Parallax, Inc.
# MIT Licensed
############################################################################
*/
#include "draw.h"
#include "text.h"

void DrawHorzChar(int col, int row, uchar *ptr2, int stride2, int scale, int color)
{
    int i, j, k, l;
    int pixel, temp;

    for (j = 0; j < CHAR_HEIGHT; j++)
    {
	for (l = 0; l < scale; l++)
	{
            temp = *ptr2;
	    for (i = 0; i < CHAR_WIDTH; i++)
	    {
                pixel = temp & 128; // Masked 1000_0000
                temp <<= 1;
	        for (k = 0; k < scale; k++)
	        {
	            if(pixel)
		    {
                        drawdot(col, row, color);
		    }
                    col++;
	        }
	    }
            row++;
            col -= CHAR_WIDTH * scale;
	}
	ptr2 += stride2;
    }
}

void DrawVertChar(int col, int row, uchar *ptr2, int stride2, int scale, int color)
{
    int i, j, k, l;
    int pixel, temp;

    for (j = 0; j < CHAR_HEIGHT; j++)
    {
	for (l = 0; l < scale; l++)
	{
            temp = *ptr2;
	    for (i = 0; i < CHAR_WIDTH; i++)
	    {
                pixel = temp & 128;
                temp <<= 1;
	        for (k = 0; k < scale; k++)
	        {
	            if(pixel)
		    {
                        drawdot(col, row, color);
		    }
                    row--;
	        }
	    }
            col++;
            row += CHAR_WIDTH * scale;
	}
	ptr2 += stride2;
    }
}

void PutCharImage(int col, int row, int value, int scale, int vert, int color)
{
    uchar *ptr2;
    static const uchar font[11][69] = {   // each indent is 69
        0x7c, 0x10, 0x78, 0x78, 0x0c, 0xfc, 0x38, 0xfe, 0x78, 0x78, 0x30, 0xfc,
        0x3c, 0xf8, 0xfe, 0xfe, 0x3c, 0xcc, 0x78, 0x1e, 0xe6, 0xf0, 0xc6, 0xc6,
        0x38, 0xfc, 0x38, 0xfc, 0x78, 0xfc, 0xcc, 0xcc, 0xc6, 0xcc, 0xcc, 0xfe,
        0x00, 0xe0, 0x00, 0x1c, 0x00, 0x38, 0x00, 0xe0, 0x18, 0x0c, 0xe0, 0x78,
        0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
        0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
        0xc6, 0x30, 0xcc, 0xcc, 0x1c, 0xc0, 0x60, 0xc6, 0xcc, 0xcc, 0x78, 0x66,
        0x66, 0x6c, 0x62, 0x66, 0x66, 0xcc, 0x30, 0x0c, 0x66, 0x60, 0xee, 0xc6,
        0x6c, 0x66, 0x6c, 0x66, 0xcc, 0xb4, 0xcc, 0xcc, 0xc6, 0xcc, 0xcc, 0xce,
        0x00, 0x60, 0x00, 0x0c, 0x00, 0x6c, 0x00, 0x60, 0x18, 0x0c, 0x60, 0x18,
        0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x20, 0x00, 0x00, 0x00, 0x00,
        0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x0c, 0x60, 0x00,
        0xce, 0xf0, 0xcc, 0x0c, 0x3c, 0xc0, 0xc0, 0xc6, 0xcc, 0xcc, 0xcc, 0x66,
        0xc6, 0x66, 0x60, 0x62, 0xc6, 0xcc, 0x30, 0x0c, 0x6c, 0x60, 0xfe, 0xe6,
        0xc6, 0x66, 0xc6, 0x66, 0xcc, 0x30, 0xcc, 0xcc, 0xc6, 0xcc, 0xcc, 0x98,
        0x00, 0x60, 0x00, 0x0c, 0x00, 0x60, 0x00, 0x60, 0x00, 0x00, 0x60, 0x18,
        0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x60, 0x00, 0x00, 0x00, 0x00,
        0x00, 0x00, 0x00, 0x00, 0x00, 0x18, 0x18, 0x30, 0x02,
        0xde, 0x30, 0x0c, 0x0c, 0x6c, 0xc0, 0xc0, 0x06, 0xec, 0xcc, 0xcc, 0x66,
        0xc0, 0x66, 0x64, 0x64, 0xc0, 0xcc, 0x30, 0x0c, 0x6c, 0x60, 0xfe, 0xf6,
        0xc6, 0x66, 0xc6, 0x66, 0xc0, 0x30, 0xcc, 0xcc, 0xc6, 0x78, 0xcc, 0x18,
        0x78, 0x7c, 0x78, 0x7c, 0x78, 0x60, 0x76, 0x6c, 0x78, 0x3c, 0x66, 0x18,
        0xfc, 0xf8, 0x78, 0xdc, 0x76, 0xec, 0x78, 0xfc, 0xcc, 0xcc, 0xc6, 0xc6,
        0x66, 0xfc, 0x00, 0x00, 0x00, 0x18, 0x30, 0x18, 0x06,
        0xd6, 0x30, 0x18, 0x38, 0xcc, 0xf8, 0xf8, 0x0c, 0x78, 0x7c, 0xcc, 0x7c,
        0xc0, 0x66, 0x7c, 0x7c, 0xc0, 0xfc, 0x30, 0x0c, 0x78, 0x60, 0xd6, 0xfe,
        0xc6, 0x7c, 0xc6, 0x7c, 0x70, 0x30, 0xcc, 0xcc, 0xd6, 0x30, 0x78, 0x30,
        0x0c, 0x66, 0xcc, 0xcc, 0xcc, 0xf8, 0xcc, 0x76, 0x18, 0x0c, 0x6c, 0x18,
        0xd6, 0xcc, 0xcc, 0x66, 0xcc, 0x6e, 0xcc, 0x60, 0xcc, 0xcc, 0xc6, 0x6c,
        0x66, 0x8c, 0x00, 0x00, 0x7e, 0x7e, 0x60, 0x0c, 0x0c,
        0xf6, 0x30, 0x30, 0x0c, 0xfe, 0x0c, 0xcc, 0x18, 0xdc, 0x18, 0xfc, 0x66,
        0xc0, 0x66, 0x64, 0x64, 0xce, 0xcc, 0x30, 0xcc, 0x6c, 0x62, 0xc6, 0xde,
        0xc6, 0x60, 0xce, 0x6c, 0x18, 0x30, 0xcc, 0xcc, 0xd6, 0x78, 0x30, 0x60,
        0x7c, 0x66, 0xc0, 0xcc, 0xfc, 0x60, 0xcc, 0x66, 0x18, 0x0c, 0x78, 0x18,
        0xd6, 0xcc, 0xcc, 0x66, 0xcc, 0x76, 0x60, 0x60, 0xcc, 0xcc, 0xd6, 0x38,
        0x66, 0x18, 0x00, 0x00, 0x00, 0x18, 0x60, 0x0c, 0x18,
        0xe6, 0x30, 0x60, 0x0c, 0x0c, 0x0c, 0xcc, 0x30, 0xcc, 0x18, 0xcc, 0x66,
        0xc6, 0x66, 0x60, 0x60, 0xc6, 0xcc, 0x30, 0xcc, 0x6c, 0x66, 0xc6, 0xce,
        0xc6, 0x60, 0xde, 0x66, 0xcc, 0x30, 0xcc, 0xcc, 0x6c, 0xcc, 0x30, 0x62,
        0xcc, 0x66, 0xc0, 0xcc, 0xc0, 0x60, 0xcc, 0x66, 0x18, 0x0c, 0x6c, 0x18,
        0xd6, 0xcc, 0xcc, 0x66, 0xcc, 0x60, 0x18, 0x60, 0xcc, 0xcc, 0xd6, 0x38,
        0x66, 0x60, 0x00, 0x00, 0x00, 0x18, 0x60, 0x0c, 0x30,
        0xc6, 0x30, 0xcc, 0xcc, 0x0c, 0xcc, 0xcc, 0x30, 0xcc, 0x30, 0xcc, 0x66,
        0x66, 0x6c, 0x62, 0x60, 0x66, 0xcc, 0x30, 0xcc, 0x66, 0x66, 0xc6, 0xc6,
        0x6c, 0x60, 0x7c, 0x66, 0xcc, 0x30, 0xcc, 0x78, 0x6c, 0xcc, 0x30, 0xc6,
        0xcc, 0x66, 0xcc, 0xcc, 0xcc, 0x60, 0x7c, 0x66, 0x18, 0x0c, 0x66, 0x18,
        0xd6, 0xcc, 0xcc, 0x66, 0xcc, 0x60, 0xcc, 0x6c, 0xcc, 0x78, 0x6c, 0x6c,
        0x3c, 0xc4, 0x38, 0x38, 0x00, 0x00, 0x30, 0x18, 0x60,
        0x7c, 0xfc, 0xfc, 0x78, 0x1e, 0x78, 0x78, 0x30, 0x78, 0x70, 0xcc, 0xfc,
        0x3c, 0xf8, 0xfe, 0xf0, 0x3e, 0xcc, 0x78, 0x78, 0xe6, 0xfe, 0xc6, 0xc6,
        0x38, 0xf0, 0x0c, 0xe6, 0x78, 0x78, 0x78, 0x30, 0x6c, 0xcc, 0x78, 0xfe,
        0x76, 0xdc, 0x78, 0x76, 0x78, 0xf0, 0x0c, 0xe6, 0x7e, 0xcc, 0xe6, 0x7e,
        0xc6, 0xcc, 0x78, 0x7c, 0x7c, 0xf0, 0x78, 0x38, 0x76, 0x30, 0x6c, 0xc6,
        0x0c, 0xfc, 0x38, 0x38, 0x00, 0x00, 0x18, 0x30, 0xc0,
        0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
        0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
        0x00, 0x00, 0x1e, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
        0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xcc, 0x00, 0x00, 0xcc, 0x00, 0x00,
        0x00, 0x00, 0x00, 0x60, 0x0c, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
        0x18, 0x00, 0x00, 0x60, 0x00, 0x00, 0x0c, 0x60, 0x80,
        0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
        0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
        0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
        0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x78, 0x00, 0x00, 0x78, 0x00, 0x00,
        0x00, 0x00, 0x00, 0xf0, 0x1e, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
        0xf0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00};

    if (value >= '0' && value <= '9')
    {
	value -= '0';
    }
    else if (value >= 'A' && value <= 'Z')
    {
	value = value - 'A' + 10;
    }
    else if (value >= 'a' && value <= 'z')
    {
	value = value - 'a' + 36;
    }
    else if (value == '.')
    {
	value = 62;
    }
    else if (value == ',')
    {
	value = 63;
    }
    else if (value == '-')
    {
	value = 64;
    }
    else if (value == '+')
    {
	value = 65;
    }
    else if (value == '(')
    {
	value = 66;
    }
    else if (value == ')')
    {
	value = 67;
    }
    else if (value == '/')
    {
	value = 68;
    }
    else return;

    ptr2 = (uchar *)&font[0][value];

    if (vert == 0)
        DrawHorzChar(col, row, ptr2, 69, scale, color);
    else
        DrawVertChar(col, row, ptr2, 69, scale, color);
}

void PutStringImage(int col, int row, char *str, int scale, int vert, int color)
{
    while (*str)
    {
	PutCharImage(col, row, str[0], scale, vert, color);
	str++;
	if (vert == 0)
	    col += CHAR_WIDTH * scale;
	else
	    row -= CHAR_WIDTH * scale;
    }
}
/*
+--------------------------------------------------------------------
|  TERMS OF USE: MIT License
+--------------------------------------------------------------------
Permission is hereby granted, free of charge, to any person obtaining
a copy of this software and associated documentation files
(the "Software"), to deal in the Software without restriction,
including without limitation the rights to use, copy, modify, merge,
publish, distribute, sublicense, and/or sell copies of the Software,
and to permit persons to whom the Software is furnished to do so,
subject to the following conditions:

The above copyright notice and this permission notice shall be
included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
+------------------------------------------------------------------
*/

